@@ -30,7 +30,7 @@ class GEO(SpaceSphericalCS):
 
     def to_APEX(self, append_mlt=False):
         cs_geoc = self.to_GEOC(kind='sph')
-        cs_new = cs_geoc.to_aacgm(append_mlt=append_mlt)
+        cs_new = cs_geoc.to_APEX(append_mlt=append_mlt)
         return cs_new
 
     def to_LENU(self, lat_0=None, lon_0=None, height_0=None, kind='car', **kwargs):
@@ -361,7 +361,7 @@ class GEOCSpherical(SpaceSphericalCS):
 
         coords = {'lat': lat*factor, 'lon': lon*factor, 'height': h}
         vector = self.vector
-        # if vector is not None:
+        # if vi is not None:
         #    raise NotImplementedError
 
         cs_new = GEO(coords=coords, vector=vector, ut=self.ut)
@@ -379,11 +379,12 @@ class GEOCSpherical(SpaceSphericalCS):
             uts = self.ut
         lat_shape = self.coords.lat.shape
         lon_shape = self.coords.lon.shape
-        if ut_type is datetime.datetime:
+        if issubclass(self.ut.__class__, datetime.datetime):
 
-            lat, lon, r = aacgm.convert_latlon_arr(in_lat=self.coords.lat.flatten(),
-                                                   in_lon=self.coords.lon.flatten(), height=self.coords.height.flatten(),
-                                                   dtime=self.ut, method_code=method_code)
+            lat, lon, r = aacgm.convert_latlon_arr(
+                in_lat=self.coords.lat.flatten(), in_lon=self.coords.lon.flatten(), height=self.coords.height.flatten(),
+                dtime=self.ut, method_code=method_code
+            )
         else:
             if uts.shape[0] != self.coords.lat.shape[0]:
                 mylog.StreamLogger.error("Datetimes must have the same length as cs!")
@@ -403,7 +404,7 @@ class GEOCSpherical(SpaceSphericalCS):
                        ut=self.ut)
         if append_mlt:
             lon = lon.flatten()
-            if ut_type is datetime.datetime:
+            if issubclass(self.ut.__class__, datetime.datetime):
                 mlt = aacgm.convert_mlt(lon, self.ut)
             else:
                 mlt = np.empty_like(lon)
@@ -425,7 +426,7 @@ class GEOCSpherical(SpaceSphericalCS):
             uts = cs_in.ut
 
         mlt = None
-        if ut_type is datetime.datetime:
+        if issubclass(self.ut.__class__, datetime.datetime):
             apex_obj = apex.Apex(cs_in.ut)
             mlat, mlon = apex_obj.convert(
                 cs_in.coords.lat, cs_in.coords.lon, 'geo', 'apex', height=cs_in.coords.height,
